/* PureAdmin
 * Copyright (C) 2003 Isak Savo
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
 * Preferences management - the GUI-part.
 *
 * Copyright (C) 2003 - Isak Savo
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <gtk/gtk.h>
#include "support.h"
#include "interface.h"
#include "cfg.h"
#include "prefwin.h"
#include "helper.h"
#include "gui_helper.h"
#include "logfile.h"
#include "globals.h"
#include "dirbrowser.h"

void prefwin_menu_gid_changed (GtkOptionMenu *optionmenu, gpointer user_data);
void prefwin_menu_uid_changed (GtkOptionMenu *optionmenu, gpointer user_data);
void prefwin_rdo_toggled (GtkToggleButton *togglebutton, gpointer user_data);
void prefwin_entry_changed (GtkEditable *editable, gpointer user_data);
void prefwin_btn_create_id (GtkButton *button, gpointer user_data);
void prefwin_btn_search_clicked (GtkButton *button, gpointer user_data);
void prefwin_chkbtn_toggled (GtkToggleButton *togglebutton, gpointer user_data);
void prefwin_chkencoding_toggled (GtkToggleButton *togglebutton, gpointer user_data);
void prefwin_btn_browse_home_clicked (GtkButton *button, gpointer user_data);
void prefwin_btn_encoding_sel_clicked (GtkButton *button, gpointer user_data);
GtkWidget *prefwin = NULL;


prefwin_widgets *prefwin_get_widgets (void)
{
	static prefwin_widgets *w = NULL;

	if (!prefwin)
		return NULL;
   
	if (!w)
	{
		w = g_new0 (prefwin_widgets, 1);
       
		w->menu_uid = lookup_widget (prefwin, "opt_menu_uid");
		w->menu_gid = lookup_widget (prefwin, "opt_menu_gid");
		w->home = lookup_widget (prefwin, "entry_home");
		w->cmd_purepw = lookup_widget (prefwin, "entry_purepw_cmd");
		w->pwfile = lookup_widget (prefwin, "entry_passwd_file");
		w->pdbfile = lookup_widget (prefwin, "entry_pdb_file");
		w->encoding = lookup_widget (prefwin, "entry_encoding");
		w->btn_search = lookup_widget (prefwin, "btn_search_set_defaults");
		w->rdo_syslog = lookup_widget (prefwin, "rdo_syslog");
		w->rdo_custom = lookup_widget (prefwin, "rdo_custom");
		w->logfile = lookup_widget (prefwin, "entry_logfile");
		w->chk_resolve = lookup_widget (prefwin, "chk_resolve_hostnames");
		w->btn_home_browse = lookup_widget (prefwin, "btn_home_dir_browse");
		w->btn_encoding_sel = lookup_widget (prefwin, "btn_encoding_sel");
		w->chk_save_window_geo = lookup_widget (prefwin, "chk_save_window_geo");
		w->chk_use_system_encoding = lookup_widget (prefwin, "chk_use_system_encoding");
	}
	return w;
}

void pref_init_prefwindow (GtkWidget *dlg)
{
	prefwin_widgets *w;
	GList *id;
	GtkWidget *menu;
	gint i;
   
	prefwin = dlg;
	w = prefwin_get_widgets ();
   
	if (!w)
	{
		g_error ("Preference window is not created!\n");
		return;
	}
	/* GID/UID Option menus */
	id = misc_get_user_ids ();
	menu = gui_create_menu_from_glist (id);
	gtk_option_menu_remove_menu (GTK_OPTION_MENU (w->menu_uid));
	gtk_option_menu_set_menu (GTK_OPTION_MENU (w->menu_uid), menu);
	g_object_set_data (G_OBJECT (w->menu_uid), "items", id);
	i = 0;
	while (id)
	{
		if (cfg.default_uid == atoi ((gchar *) id->data))
			break;
		i++;
		id = g_list_next (id);
	}
	gtk_option_menu_set_history (GTK_OPTION_MENU (w->menu_uid), i);

	id = misc_get_group_ids ();
	menu = gui_create_menu_from_glist (id);
	gtk_option_menu_remove_menu (GTK_OPTION_MENU (w->menu_gid));
	gtk_option_menu_set_menu (GTK_OPTION_MENU (w->menu_gid), menu);
	g_object_set_data (G_OBJECT (w->menu_gid), "items", id);
	i = 0;
	while (id)
	{
		if (cfg.default_gid == atoi ((gchar *) id->data))
			break;
		i++;
		id = g_list_next (id);
	}
	gtk_option_menu_set_history (GTK_OPTION_MENU (w->menu_gid), i);

	if (cfg.logmethod == LOG_SYSLOG)
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w->rdo_syslog), TRUE);
	else
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w->rdo_custom), TRUE);
   
	gtk_entry_set_text (GTK_ENTRY (w->home), cfg.default_home);
	gtk_entry_set_text (GTK_ENTRY (w->encoding), cfg.uname_encoding);
	gtk_entry_set_text (GTK_ENTRY (w->cmd_purepw), cfg.cmd_purepw);
	gtk_entry_set_text (GTK_ENTRY (w->pwfile), cfg.pwfile);
	gtk_entry_set_text (GTK_ENTRY (w->pdbfile), cfg.pdbfile);
	gtk_entry_set_text (GTK_ENTRY (w->logfile), cfg.logfile);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w->chk_resolve), cfg.resolve_hostnames);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w->chk_save_window_geo), cfg.save_window_geometry);
	g_signal_connect ((gpointer) w->chk_use_system_encoding, "toggled",
			  G_CALLBACK (prefwin_chkencoding_toggled),
			  NULL);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w->chk_use_system_encoding), cfg.use_system_encoding);
	/* Hookup signals for the widgets to allow 'instant-apply' */
	g_signal_connect ((gpointer) w->menu_uid, "changed",
			  G_CALLBACK (prefwin_menu_uid_changed),
			  NULL);
	g_signal_connect ((gpointer) w->menu_gid, "changed",
			  G_CALLBACK (prefwin_menu_gid_changed),
			  NULL);
	g_signal_connect ((gpointer) w->home, "changed",
			  G_CALLBACK (prefwin_entry_changed),
			  "home");
	g_signal_connect ((gpointer) w->encoding, "changed",
			  G_CALLBACK (prefwin_entry_changed),
			  "encoding");
	g_signal_connect ((gpointer) w->cmd_purepw, "changed",
			  G_CALLBACK (prefwin_entry_changed),
			  "cmd_purepw");
	g_signal_connect ((gpointer) w->pwfile, "changed",
			  G_CALLBACK (prefwin_entry_changed),
			  "pwfile");
	g_signal_connect ((gpointer) w->pdbfile, "changed",
			  G_CALLBACK (prefwin_entry_changed),
			  "pdbfile");
	g_signal_connect ((gpointer) w->rdo_syslog, "toggled",
			  G_CALLBACK (prefwin_rdo_toggled),
			  GINT_TO_POINTER (LOG_SYSLOG));
	g_signal_connect ((gpointer) w->rdo_custom, "toggled",
			  G_CALLBACK (prefwin_rdo_toggled),
			  GINT_TO_POINTER (LOG_CUSTOM));
	g_signal_connect ((gpointer) w->logfile, "changed",
			  G_CALLBACK (prefwin_entry_changed),
			  "logfile");
	g_signal_connect ((gpointer) w->chk_resolve, "toggled",
			  G_CALLBACK (prefwin_chkbtn_toggled),
			  "resolve");

	g_signal_connect ((gpointer) w->chk_save_window_geo, "toggled",
			  G_CALLBACK (prefwin_chkbtn_toggled),
			  "save_window");
   
	/* Button Callbacks */
	g_signal_connect ((gpointer) w->btn_search, "clicked",
			  G_CALLBACK (prefwin_btn_search_clicked),
			  NULL);
	g_signal_connect ((gpointer) w->btn_home_browse, "clicked",
			  G_CALLBACK (prefwin_btn_browse_home_clicked),
			  NULL);
	g_signal_connect ((gpointer) w->btn_encoding_sel, "clicked",
			  G_CALLBACK (prefwin_btn_encoding_sel_clicked),
			  NULL);

}

void prefwin_menu_gid_changed (GtkOptionMenu *optionmenu, gpointer user_data)
{
	GList *id;
	gint node_no;
   
	id = (GList *) g_object_get_data (G_OBJECT (optionmenu), "items");
	node_no = gtk_option_menu_get_history (optionmenu);
	id = g_list_nth (id, node_no);
	if (!id)
		g_print ("Gid change: Wrong history: %d\n", node_no);
}

void prefwin_menu_uid_changed (GtkOptionMenu *optionmenu, gpointer user_data)
{
	GList *id;
	gint node_no;
   
	id = (GList *) g_object_get_data (G_OBJECT (optionmenu), "items");
	node_no = gtk_option_menu_get_history (optionmenu);
	id = g_list_nth (id, node_no);
	if (!id)
		g_print ("Uid change: Wrong history: %d\n", node_no);
}

void prefwin_rdo_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	GtkWidget *entry;
   
	if (!gtk_toggle_button_get_active (togglebutton))
		return;
	entry = lookup_widget (prefwin, "entry_logfile");
   
	cfg.logmethod = GPOINTER_TO_INT (user_data);
	switch (GPOINTER_TO_INT (user_data))
	{
	case LOG_SYSLOG: /* Syslog selected */
		gtk_entry_set_text (GTK_ENTRY (entry), "/var/log/messages");
		break;
	case LOG_CUSTOM: /* Custom selected */
		gtk_entry_set_text (GTK_ENTRY (entry), "/var/log/pureftpd.log");
		break;
	}
}

void prefwin_chkbtn_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	gchar *arg = (gchar *) user_data;
	
	if (strncmp (arg, "resolve", 7) == 0)
		cfg.resolve_hostnames = gtk_toggle_button_get_active (togglebutton);
	else if (strncmp (arg, "save_window", 11) == 0)
		cfg.save_window_geometry = gtk_toggle_button_get_active (togglebutton);
}

void prefwin_chkencoding_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	prefwin_widgets *w;
	gboolean active;
	
	w = prefwin_get_widgets ();
	active = gtk_toggle_button_get_active (togglebutton);
	cfg.use_system_encoding = active;
	gtk_widget_set_sensitive (w->encoding, !active);
	gtk_widget_set_sensitive (w->btn_encoding_sel, !active);
}


void prefwin_entry_changed (GtkEditable *editable, gpointer user_data)
{
	gchar *tmp;
	gchar *ename;
	ename = (gchar *) user_data;
	tmp = gtk_editable_get_chars (GTK_EDITABLE (editable), 0, -1);
   
	/*    g_print ("Entry [%s] changed into: %s\n", ename, tmp); */
	if (g_str_equal (ename, "home"))
	{
		g_free (cfg.default_home);
		cfg.default_home = tmp;
	}
	else if (g_str_equal (ename, "cmd_purepw"))
	{
		g_free (cfg.cmd_purepw);
		cfg.cmd_purepw = tmp;
	}
	else if (g_str_equal (ename, "pwfile"))
	{
		g_free (cfg.pwfile);
		cfg.pwfile = tmp;
	} 
	else if (g_str_equal (ename, "pdbfile"))
	{
		g_free (cfg.pdbfile);
		cfg.pdbfile = tmp;
	}
	else if (g_str_equal(ename, "logfile"))
	{
		if (g_path_is_absolute (tmp) && g_file_test (tmp, G_FILE_TEST_IS_REGULAR))
		{
			close_logfile ();
			g_free (cfg.logfile);
			cfg.logfile = tmp;
			init_logfile ();
		}
		else
			g_free (tmp);
	}
	else if (g_str_equal (ename, "encoding"))
	{
		g_free (cfg.uname_encoding);
		cfg.uname_encoding = tmp;
	}
	else
		g_free (tmp);
}

void prefwin_btn_search_clicked (GtkButton *button, gpointer user_data)
{
	prefwin_widgets *w;

	w = prefwin_get_widgets ();
   
	gtk_entry_set_text (GTK_ENTRY (w->cmd_purepw), srv_vars.cmd_pure_pw);
	gtk_entry_set_text (GTK_ENTRY (w->pwfile), srv_vars.fl_pwfile);
	gtk_entry_set_text (GTK_ENTRY (w->pdbfile), srv_vars.fl_pdbfile);
}

void prefwin_btn_browse_home_clicked (GtkButton *button, gpointer user_data)
{
	prefwin_widgets *w;
	GtkWidget *dbr;
	const gchar *dir;
	gint response;
   
	w = prefwin_get_widgets ();
	dbr = dirbrowser_new (_("Select Default Home Directory"));
	if (g_file_test (cfg.default_home, G_FILE_TEST_IS_DIR))
		dirbrowser_set_directory (DIRBROWSER (dbr), cfg.default_home);

	response = gtk_dialog_run (GTK_DIALOG (dbr));
	if (response == GTK_RESPONSE_OK)
	{
		dir = dirbrowser_get_directory (DIRBROWSER (dbr));
		gtk_entry_set_text (GTK_ENTRY (w->home), dir);
	}
	gtk_widget_destroy (dbr);
}

static void
encoding_changed_cb (GtkTreeView *tree, gpointer user_data)
{
	GtkEntry *entry = GTK_ENTRY (user_data);
	GtkTreeModel *model;
	GtkTreeSelection *sel;
	GtkTreeIter iter;
	gchar *encoding;
	g_print ("encoding changed\n");
	sel = gtk_tree_view_get_selection (tree);
	model = gtk_tree_view_get_model (tree);
	if (gtk_tree_selection_get_selected (sel, &model, &iter))
	{
		gtk_tree_model_get (model, &iter, 
				    0, &encoding,
				    -1);
		gtk_entry_set_text (entry, encoding);
	}
	return;
}

static GList *
get_available_encodings (void)
{
	GList *retval = NULL;
#define BUFSIZE 256
	gchar buf[BUFSIZE], *tmp;
	gchar *prgm, *full_prgm;
	FILE *p;

	if (!(prgm =g_find_program_in_path ("iconv"))) {
		pur_log_wrn ("Couldn't find the program \"iconv\". Unable to fetch list of encodings");
		return NULL;
	}
	
	full_prgm = g_strdup_printf ("%s --list", prgm);
	g_free (prgm);
	if ((p = popen (full_prgm, "r")) == NULL)
	{
		pur_log_wrn ("popen %s: %s", full_prgm, strerror(errno));
		return NULL;
	}
	g_free (full_prgm);
	while (fgets (buf, BUFSIZE, p))
	{
		if (!strstr (buf, "//"))
			continue;
		tmp = buf;
		while (tmp){
			if (*tmp == '/' && *(tmp +1) == '/') {
				*tmp = '\0';
				break;
			}
			tmp++;
		}
		g_strstrip (buf);
		retval = g_list_append (retval, g_strdup (buf));
	}
	pclose (p);

	return retval;
}
static void
free_encoding_list (GtkWidget *w, gpointer list)
{
	g_list_free ((GList *) list);
}

static void
filter_encoding_list (GtkWidget *entry, gpointer data)
{
	GtkTreeView *tree = GTK_TREE_VIEW (data);
	GtkTreeIter iter, iter_sel;
	GtkListStore *model;
	GList *list;
	gint handler_id;
	const gchar *filter_str, *encoding;
	gboolean sel = FALSE;

	filter_str = gtk_entry_get_text (GTK_ENTRY(entry));
	handler_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (tree), "callback_handler_id"));
	list = g_object_get_data (G_OBJECT (tree), "encoding_list");
	model = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (tree)));
	
	g_signal_handler_block (G_OBJECT (tree), handler_id);
	gtk_list_store_clear (GTK_LIST_STORE (model));
	while (list)
	{
		encoding = (const gchar *) list->data;
		if (misc_str_isearch (encoding, filter_str))
		{
			gtk_list_store_append (GTK_LIST_STORE (model), &iter);
			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
					    0, encoding,
					    -1);
			if (g_str_equal (encoding, cfg.uname_encoding)) {
				memcpy (&iter_sel, &iter, sizeof (GtkTreeIter));
				sel = TRUE;
			}
		}
		list = g_list_next (list);
		
	}
	if (sel)
		gui_select_treeview_row (&iter_sel, tree);
		
	g_signal_handler_unblock (G_OBJECT (tree), handler_id);

	return;
}

void
prefwin_btn_encoding_sel_clicked (GtkButton *button, gpointer user_data)
{
	prefwin_widgets *w;
	static GtkWidget *dlg = NULL;
	GList *list, *head;
	GtkListStore *model;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkTreeIter iter, iter_sel;
	GtkTreeView *tree;
	GtkWidget *entry;
	guint handler_id;
	gboolean sel = FALSE;
	
	if (dlg) {
		gtk_window_present (GTK_WINDOW (dlg));
		return;
	}
	w = prefwin_get_widgets ();
	dlg = create_dlg_encoding ();
	tree = GTK_TREE_VIEW (lookup_widget (dlg, "tree_encoding"));
	model  = gtk_list_store_new (1, G_TYPE_STRING);
   
	gtk_tree_view_set_model (GTK_TREE_VIEW (tree), GTK_TREE_MODEL (model));
	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (tree)),
				     GTK_SELECTION_SINGLE);

	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes (NULL, renderer,
							   "text", 0,
							   NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW(tree), column);

	head = list = get_available_encodings ();

	while (list)
	{
		gtk_list_store_append (GTK_LIST_STORE (model), &iter);
		gtk_list_store_set (GTK_LIST_STORE (model), &iter,
				    0, (gchar *) list->data,
				    -1);
		if (g_str_equal ((gchar*) list->data, cfg.uname_encoding)) {
			memcpy (&iter_sel, &iter, sizeof (GtkTreeIter));
			sel = TRUE;
		}
		
		list = g_list_next (list);
	}
	if (sel)
		gui_select_treeview_row (&iter_sel, tree);
	
	
	entry = lookup_widget (dlg, "entry_filter");
	
	g_signal_connect (G_OBJECT (entry), "changed",
			  G_CALLBACK (filter_encoding_list), (gpointer) tree);
	
	handler_id = g_signal_connect (G_OBJECT (tree), "cursor-changed",
				       G_CALLBACK (encoding_changed_cb), w->encoding);
	g_object_set_data (G_OBJECT (tree), "callback_handler_id", GINT_TO_POINTER (handler_id));
	g_object_set_data (G_OBJECT (tree), "encoding_list", head);
	g_signal_connect (G_OBJECT (dlg), "destroy",
			  G_CALLBACK (free_encoding_list), (gpointer) head);
	
	gtk_dialog_run (GTK_DIALOG (dlg));
	
	gtk_widget_destroy (dlg);
	dlg = NULL;
}
