/* PureAdmin
 * Copyright (C) 2003 Isak Savo
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
 * Functions related to run a program as root
 *
 * Copyright (C) 2003-2004 Isak Savo
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef HAVE_LIBDL
#include <dlfcn.h>
#endif

#include <gtk/gtk.h>
#include "globals.h"
#include "prefix.h"

#ifndef SELFPATH
# define SELFPATH BINDIR G_DIR_SEPARATOR_S "pureadmin"
#endif

static gpointer gnomesu_dl = NULL;

#ifndef HAVE_LIBDL
gboolean init_suexec (void){
	return FALSE;
}
gint run_as_root (gchar **argv){
	return 0;
}

void suexec_shutdown (void) {
	return;
}
#else 

/* init_suexec: Tries to dlopen() libgnomesu, enabling easy switch-user
 * functions.
 */
gboolean init_suexec (void)
{
    if (gnomesu_dl == NULL)
    {
	gnomesu_dl = dlopen ("libgnomesu.so", RTLD_LAZY);
	if (!gnomesu_dl)
	  gnomesu_dl = dlopen ("libgnomesu.so.0", RTLD_LAZY); /* One more try */
	if (!gnomesu_dl)
	  return FALSE;
    }
    return TRUE;
}

/* suexec_shutdown: Closes the handle to libgnomesu */
void suexec_shutdown (void)
{
    if (!gnomesu_dl)
      return;
    if (dlclose(gnomesu_dl) != 0)
      pur_log_wrn ("Unable to close gnomesu dl-handle: %s", dlerror());
}

/* run_as_root: Wrapper around gnomesu_spawn_async() that executes pureadmin
 *		as root. It does some sanity checks first, and makes sure that
 *		the program executed is an absolute path.
 *
 * Arguments: argv	The argument vector from main()
 */
gint run_as_root (gchar **argv)
{
    static gboolean (*spawn_command) (gchar *,gchar**, gint*) = NULL;
    gchar *prgm, *tmp;
    gint pid = 0;
        
    if (!init_suexec())
      return 0;
    dlerror();

    if (spawn_command == NULL)
    {
	spawn_command = dlsym (gnomesu_dl, "gnomesu_spawn_async");
	if ((tmp = dlerror()) != NULL) {
	    pur_log_wrn ("Unable to retreive gnomesu function: %s", tmp);
	    return 0;
	}
    }
    /* Ensure we have full path to pureadmin! */
    if (!g_path_is_absolute (SELFPATH)) {
	pur_log_wrn ("Cannot run %s: not an absolute path", SELFPATH);
	return 0;
    }
    
    prgm = g_strdup (SELFPATH);
    tmp = argv[0];
    argv[0] = prgm;

    if (!spawn_command ("root", argv ,&pid))
      pur_log_wrn ("Unable to run pureadmin as root");

    argv[0] = tmp;
    g_free(prgm);
    return pid;
}

#endif /* HAVE_LIBDL */
